const Team = {
  X: 0,
  O: 1,
  properties: {
    0: {name: "X"},
    1: {name: "O"}
  }
};

function undisplay(element) {
  element.style.display="none";
}

function add_message(severity, message) {
  message = `<div class="message ${severity}" onclick="undisplay(this)">
<span class="hide-button" onclick="undisplay(this.parentElement)">&times;</span>
${message}
</div>`;
  const message_area = document.getElementById('message-area');
  message_area.insertAdjacentHTML('beforeend', message);
}

const events = new EventSource("events");

events.onerror = function(event) {
  if (event.target.readyState === EventSource.CLOSED) {
      add_message("danger", "Connection to server lost.");
  }
};

events.addEventListener("move", event => {
  const move = JSON.parse(event.data);

  window.game.receiveMove(move);
});

events.addEventListener("game-state", event => {
  const state = JSON.parse(event.data);

  window.game.resetState();

  for (let square of state.moves) {
    window.game.receiveMove(square);
  }
});

function Square(props) {
  let className = "square";

  if (props.value) {
    className += " occupied";
  } else if (props.active) {
    className += " open";
  }

  const onClick = props.active ? props.onClick : null;

  return (
    <div className={className}
         onClick={onClick}>
      {props.value}
    </div>
  );
}

class Board extends React.Component {
  renderSquare(i) {
    const value = this.props.squares[i];
    return (
      <Square
        value={value}
        active={! this.props.gameOver && ! value}
        onClick={() => this.props.onClick(i)}
      />
    );
  }

  render() {
    return (
      <div>
        <div className="board-row">
          {this.renderSquare(0)}
          {this.renderSquare(1)}
          {this.renderSquare(2)}
        </div>
        <div className="board-row">
          {this.renderSquare(3)}
          {this.renderSquare(4)}
          {this.renderSquare(5)}
        </div>
        <div className="board-row">
          {this.renderSquare(6)}
          {this.renderSquare(7)}
          {this.renderSquare(8)}
        </div>
      </div>
    );
  }
}

function fetch_post_json(api = '', data = {}) {
  const response = fetch(api, {
    method: 'POST',
    headers: {
      'Content-Type': 'application/json'
    },
    body: JSON.stringify(data)
  });
  return response;
}

class Game extends React.Component {
  constructor(props) {
    super(props);
    this.state = {
      history: [
        {
          squares: Array(9).fill(null)
        }
      ],
      step_number: 0,
      next_to_play: Team.X
    };
  }

  sendMove(i) {
    return fetch_post_json("move", { move: i });
  }

  resetState() {
    this.setState({
      history: [
        {
          squares: Array(9).fill(null)
        }
      ],
      step_number: 0,
      next_to_play: Team.X
    });
  }

  receiveMove(i) {
    const history = this.state.history.slice(0, this.state.step_number + 1);
    const current = history[history.length - 1];
    const squares = current.squares.slice();
    if (calculateWinner(squares) || squares[i]) {
      return;
    }
    squares[i] = Team.properties[this.state.next_to_play].name;
    let next_to_play;
    if (this.state.next_to_play === Team.X)
      next_to_play = Team.O;
    else
      next_to_play = Team.X;
    this.setState({
      history: history.concat([
        {
          squares: squares
        }
      ]),
      step_number: history.length,
      next_to_play: next_to_play
    });
  }

  async handleClick(i) {
    const response = await this.sendMove(i);
    if (response.status == 200) {
      const result = await response.json();
      if (! result.legal)
        add_message("danger", result.message);
    } else {
      add_message("danger", `Error occurred sending move`);
    }
  }

  render() {
    const history = this.state.history;
    const current = history[this.state.step_number];
    const winner = calculateWinner(current.squares);

    let status;
    if (winner) {
      status = "Winner: " + winner;
    } else {
      status = "Next player: " + (Team.properties[this.state.next_to_play].name);
    }

    return (
      <div className="game">
        <div className="game-info">
          <div>{status}</div>
        </div>
        <div className="game-board">
          <Board
            gameOver={winner}
            squares={current.squares}
            onClick={i => this.handleClick(i)}
          />
        </div>
      </div>
    );
  }
}

// ========================================

ReactDOM.render(<Game
                  ref={(me) => window.game = me}
                />, document.getElementById("tictactoe"));

function calculateWinner(squares) {
  const lines = [
    [0, 1, 2],
    [3, 4, 5],
    [6, 7, 8],
    [0, 3, 6],
    [1, 4, 7],
    [2, 5, 8],
    [0, 4, 8],
    [2, 4, 6]
  ];
  for (let i = 0; i < lines.length; i++) {
    const [a, b, c] = lines[i];
    if (squares[a] && squares[a] === squares[b] && squares[a] === squares[c]) {
      return squares[a];
    }
  }
  return null;
}
